﻿using ServiceCenter.Extensions;
using ServiceCenter.Logs;
using ServiceCenter.SqlSugars;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using WCS.Core;
using WCS.Entity;
using WCS.WorkEngineering.Extensions;
using WCS.WorkEngineering.Protocol.Station;
using WCS.WorkEngineering.WebApi.Controllers;
using WCS.WorkEngineering.Worlds;
using DeviceFlags = WCS.WorkEngineering.Extensions.DeviceFlags;

namespace WCS.WorkEngineering.Systems
{
    /// <summary>
    ///  出库任务处理
    /// </summary>
    [BelongTo(typeof(MainWorld))]
    [Description("出库任务处理")]
    public class 出库任务处理 : DeviceSystem<Device<IStation520, IStation521, IStation523>>
    {
        protected override bool ParallelDo => false;
        protected override bool SaveLogsToFile => true;
        List<Station> devs = new List<Station>();
        public 出库任务处理()
        {
            devs = Device.All.Where(v => v.HasFlag(DeviceFlags.出货口)).Select(v => new Station(v, this.World)).ToList();
        }

        public override void Do(Device<IStation520, IStation521, IStation523> obj)
        {
            if (obj.Data2.Request != 1) 
            {
                if (obj.Data2.GoodsEnd != "8269".ToShort())
                {
                    throw new KnownException("无请求", LogLevelEnum.Mid);
                }
            } 
            if (obj.Data2.TaskNumber <= 0)
                return;
            if (obj.Data2.GoodsEnd != obj.Entity.Code.ToShort()) return;

            SqlSugarHelper.Do(db =>
            {
                //找到对应任务
                var taskInfo = db.Default.Queryable<WCS_TaskInfo>().First(v => v.ID == obj.Data2.TaskNumber);
                if (taskInfo == null) throw new KnownException($"未找到对应WCS任务{obj.Data2.TaskNumber}", LogLevelEnum.High);
                if (taskInfo.AddrTo == obj.Entity.Code && taskInfo.Type == TaskType.OutDepot && taskInfo.AddrTo != "8278")
                {
                    taskInfo.Status = Entity.TaskStatus.Finish;
                    taskInfo.EditTime = DateTime.Now;
                    taskInfo.EedTime = DateTime.Now;

                    db.Default.Updateable(taskInfo).ExecuteCommand();
                    taskInfo.AddWCS_TASK_DTL(db, obj.Entity.Code, taskInfo.AddrNext, $"出库任务完成");
                    WmsApi.CompleteTask(taskInfo.ID);
                    taskInfo.CompleteOrCancelTasks(db);
                    if ( (obj.Entity.Code == "8263" || obj.Entity.Code == "8257"))
                    {
                        obj.Data.TaskNumber = taskInfo.ID;
                        obj.Data.GoodsStart = Convert.ToInt16(obj.Entity.Code);
                        obj.Data.GoodsEnd = 8262;
                        obj.Data.VoucherNo++;
                        World.Log($"出库位置{obj.Entity.Code}任务{obj.Data2.TaskNumber}完成，写入PLC凭证号{obj.Data.VoucherNo+1}", LogLevelEnum.Mid);
                        return;
                    }
                    else if(obj.Entity.Code == "8269" || obj.Entity.Code == "8278")
                    {
                        obj.Data.TaskNumber = 0;
                        obj.Data.VoucherNo++;
                        World.Log($"出库位置{obj.Entity.Code}任务{obj.Data2.TaskNumber}完成，写入PLC凭证号{obj.Data.VoucherNo+1}", LogLevelEnum.Mid);
                        return;
                    }
                }
            }); 
        }
        public override bool Select(Device dev)
        {
            return dev.HasFlag(DeviceFlags.出货口);
        }
    }
}
