﻿using ServiceCenter.Extensions;
using ServiceCenter.Logs;
using ServiceCenter.SqlSugars;
using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using WCS.Core;
using WCS.Entity;
using WCS.WorkEngineering.Extensions;
using WCS.WorkEngineering.Protocol.BCR;
using WCS.WorkEngineering.Protocol.SRM;
using WCS.WorkEngineering.Protocol.Station;
using WCS.WorkEngineering.WebApi.Controllers;
using WCS.WorkEngineering.Worlds;
using static System.Runtime.InteropServices.JavaScript.JSType;
using DeviceFlags = WCS.WorkEngineering.Extensions.DeviceFlags;

namespace WCS.WorkEngineering.Systems
{
    /// <summary>
    ///  装箱区子母托盘输送
    /// </summary>
    [BelongTo(typeof(MainWorld))]
    [Description("装箱区子母托盘输送")]
    public class 装箱区子母托盘输送 : DeviceSystem<Device<IStation520, IStation521, IStation523>>
    {
        protected override bool ParallelDo => true;
        protected override bool SaveLogsToFile => true;
        private List<Station> devs = new List<Station>();
        private List<SRM> srms = new List<SRM>();
        public override void Do(Device<IStation520, IStation521, IStation523> obj)
        {
            obj.入库站点是否被禁止();
            obj.入库站点是否满足执行条件();

            WCS_TaskInfo task = null;
            SqlSugarHelper.Do(_db =>
            {
                var db = _db.Default;
                //找到对应任务
                var taskInfo = db.Queryable<WCS_TaskInfo>().First(v=>v.AddrFrom== obj.Entity.Code && v.Status == Entity.TaskStatus.WaitingToExecute);
                //未找到对应任务
                if (taskInfo == null)
                {
                    return;
                }
                
                    taskInfo.Status = Entity.TaskStatus.ConveyorExecution;
                    taskInfo.EditWho = "WCS";
                    taskInfo.EditTime = DateTime.Now;
                    taskInfo.AddrNext = "8058";
                    db.Updateable(taskInfo).ExecuteCommand();
                    taskInfo.AddWCS_TASK_DTL(_db, obj.Entity.Code, taskInfo.AddrNext, $"子托盘开始输送，目标地址：{taskInfo.AddrTo}");
                    //taskInfo.updateRedisHash();
                    task = taskInfo;
                
            });
            if (task == null) return;
            obj.Data.TaskNumber = task.ID;
            obj.Data.GoodsStart = obj.Entity.Code.ToShort();
            obj.Data.GoodsEnd = task.AddrNext.ToShort();
            obj.Data.VoucherNo++;

        }
        public override bool Select(Device dev)
        {
            return dev.Code == "8003" || dev.Code == "8011"|| dev.Code == "8019";
        }
    }
}
