﻿using ServiceCenter.Extensions;
using ServiceCenter.Logs;
using ServiceCenter.SqlSugars;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using WCS.Core;
using WCS.Entity;
using WCS.WorkEngineering.Extensions;
using WCS.WorkEngineering.Protocol.Station;
using WCS.WorkEngineering.WebApi.Controllers;
using WCS.WorkEngineering.Worlds;
using DeviceFlags = WCS.WorkEngineering.Extensions.DeviceFlags;

namespace WCS.WorkEngineering.Systems
{
    /// <summary>
    ///  装箱区打包位置分配
    /// </summary>
    [BelongTo(typeof(MainWorld))]
    [Description("装箱区打包位置分配")]
    public class 装箱区打包位置分配 : DeviceSystem<Device<IStation520, IStation521, IStation523>>
    {
        protected override bool ParallelDo => true;
        protected override bool SaveLogsToFile => true;
        List<Station> devs = new List<Station>();

        public override void Do(Device<IStation520, IStation521, IStation523> obj)
        {
            obj.入库站点是否满足执行条件();
            WCS_TaskInfo task = null;
            SqlSugarHelper.Do(db =>
            {
                //找到对应任务
                var taskInfo = db.Default.Queryable<WCS_TaskInfo>().First(v => v.ID == obj.Data2.TaskNumber);
                if (taskInfo == null) throw new KnownException($"未找到对应WCS任务{obj.Data2.TaskNumber}", LogLevelEnum.High);

                taskInfo.Status = Entity.TaskStatus.ConveyorExecution;
                taskInfo.EditTime = DateTime.Now;
                taskInfo.EedTime = DateTime.Now;
                if (taskInfo.AddrTo == "8148")
                {
                    taskInfo.AddrNext = "8148";
                }
                else
                {
                    devs = Device.All.Where(v => v.Code is "8139" or "8132").Select(v => new Station(v, this.World)).ToList();
                    devs = devs.Where(v => !v.Data3.Status.HasFlag(StationStatus.PH_Status) //无光电
                                              && !v.Data3.Status.HasFlag(StationStatus.Run) //未运行
                                              && !v.Data3.Status.HasFlag(StationStatus.OT_Status) //无任务
                                              && !v.Data3.Status.HasFlag(StationStatus.UnassignedTask) //未分配任务
                                              && v.Data3.Status.HasFlag(StationStatus.Auto)//自动
                                              && !db.Default.Queryable<WCS_TaskInfo>().Any(w => w.AddrNext == v.Entity.Code)
                                              ).ToList();
                    if (!devs.Any()) throw new KnownException($"[{obj.Entity.Code}]无可用打包站台", LogLevelEnum.Mid);
                    var dev = devs.OrderByDescending(v => v.Entity.Code.ToShort()).FirstOrDefault();

                    taskInfo.AddrNext = dev.Entity.Code;
                }
                db.Default.Updateable(taskInfo).ExecuteCommand();
                task = taskInfo;
            });
            if (task == null) return;
            obj.Data.TaskNumber = task.ID;
            obj.Data.GoodsStart = obj.Entity.Code.ToShort();
            obj.Data.GoodsEnd = task.AddrNext.ToShort();
            obj.Data.VoucherNo++;
        }
       
        public override bool Select(Device dev)
        {
            return dev.Code == "8128";
        }
    }
}
