﻿using ServiceCenter.Extensions;
using ServiceCenter.Logs;
using ServiceCenter.SqlSugars;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using WCS.Core;
using WCS.Entity;
using WCS.WorkEngineering.Extensions;
using WCS.WorkEngineering.Protocol.Station;
using WCS.WorkEngineering.WebApi.Controllers;
using WCS.WorkEngineering.Worlds;
using DeviceFlags = WCS.WorkEngineering.Extensions.DeviceFlags;

namespace WCS.WorkEngineering.Systems
{
    /// <summary>
    ///  装箱区打包工位
    /// </summary>
    [BelongTo(typeof(MainWorld))]
    [Description("装箱区打包工位")]
    public class 装箱区打包工位 : DeviceSystem<Device<IStation520, IStation521, IStation523>>
    {
        protected override bool ParallelDo => true;
        protected override bool SaveLogsToFile => true;
        List<Station> devs = new List<Station>();
        public override void Do(Device<IStation520, IStation521, IStation523> obj)
        {
            obj.入库站点是否满足执行条件();
            WCS_TaskInfo task = null;
            SqlSugarHelper.Do(db =>
            {
                //找到对应任务
                var taskInfo = db.Default.Queryable<WCS_TaskInfo>().First(v => v.ID == obj.Data2.TaskNumber);
                if (taskInfo == null) throw new KnownException($"未找到对应WCS任务{obj.Data2.TaskNumber}", LogLevelEnum.High);

                if(taskInfo.AddrTo=="8148") throw new KnownException($"异常口任务{obj.Data2.TaskNumber}，不能入库", LogLevelEnum.High);
                taskInfo.EditTime = DateTime.Now;
                taskInfo.EedTime = DateTime.Now;
                //taskInfo.AddrNext ="8300";
                taskInfo.AddrNext = "8058";
                taskInfo.AddrTo ="SRM";
                db.Default.Updateable(taskInfo).ExecuteCommand();
                task = taskInfo;
            });
            if (task == null) return;
            obj.Data.TaskNumber = task.ID;
            obj.Data.GoodsStart = obj.Entity.Code.ToShort();
            obj.Data.GoodsEnd = task.AddrNext.ToShort();
            obj.Data.VoucherNo++;
        }
       
        public override bool Select(Device dev)
        {
            //return dev.Code == "8132"|| dev.Code == "8137"|| dev.Code == "8142" || dev.Code == "8145";
            //return dev.Code == "8132"|| dev.Code == "8139" || dev.Code == "8148";
            return dev.Code == "8132"|| dev.Code == "8139" || dev.Code == "8148";
        }
    }
}
