﻿using ServiceCenter.Extensions;
using ServiceCenter.Logs;
using ServiceCenter.SqlSugars;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using WCS.Core;
using WCS.Entity;
using WCS.WorkEngineering.Extensions;
using WCS.WorkEngineering.Protocol.Station;
using WCS.WorkEngineering.WebApi.Controllers;
using WCS.WorkEngineering.Worlds;
using DeviceFlags = WCS.WorkEngineering.Extensions.DeviceFlags;

namespace WCS.WorkEngineering.Systems
{
    /// <summary>
    ///  装箱区码垛工位任务处理
    /// </summary>
    [BelongTo(typeof(MainWorld))]
    [Description("装箱区码垛工位任务处理")]
    public class 装箱区码垛工位任务处理 : DeviceSystem<Device<IStation520, IStation521, IStation523>>
    {
        protected override bool ParallelDo => true;
        protected override bool SaveLogsToFile => true;
        List<Station> devs = new List<Station>();
        public override void Do(Device<IStation520, IStation521, IStation523> obj)
        {
            WCS_TaskInfo task = null;
            SqlSugarHelper.Do(db =>
            {
                //找到对应任务
                var taskInfo = db.Default.Queryable<WCS_TaskInfo>().First(v => v.ID == obj.Data2.TaskNumber);
                if (obj.Entity.Code == "8307")
                {
                    World.Log($"{obj.Entity.Code}id{obj.Data2.TaskNumber}");
                }
                
                if (taskInfo != null && taskInfo.AddrTo == obj.Entity.Code && taskInfo.Status != Entity.TaskStatus.Finish)
                {
                    taskInfo.Status = Entity.TaskStatus.Finish;
                    taskInfo.EditTime = DateTime.Now;
                    taskInfo.EedTime = DateTime.Now;
                    db.Default.Updateable(taskInfo).ExecuteCommand();
                    taskInfo.AddWCS_TASK_DTL(db, obj.Entity.Code, taskInfo.AddrNext, $"任务完成");
                    taskInfo.CompleteTasks(db);
                }
                else
                {
                    obj.码垛工位是否满足执行条件();

                    //找到对应任务
                    taskInfo = db.Default.Queryable<WCS_TaskInfo>().First(v => v.AddrFrom == obj.Entity.Code && v.Status == Entity.TaskStatus.WaitingToExecute);
                    if (taskInfo == null) throw new KnownException($"该位置暂无任务", LogLevelEnum.High);

                    taskInfo.Status = Entity.TaskStatus.ConveyorExecution;
                    taskInfo.EditTime = DateTime.Now;
                    taskInfo.EedTime = DateTime.Now;
                    taskInfo.AddrNext = "8128";

                    db.Default.Updateable(taskInfo).ExecuteCommand();
                    if (taskInfo.AddrTo == "8148")
                        taskInfo.AddWCS_TASK_DTL(db, obj.Entity.Code, taskInfo.AddrNext, $"异常口任务，准备输送到异常口");
                    else
                        taskInfo.AddWCS_TASK_DTL(db, obj.Entity.Code, taskInfo.AddrNext, $"码垛完成，任务开始");

                    task = taskInfo;
                }
            });
            SqlSugarHelper.Do(db =>
            {
                var taskInfo = db.Default.Queryable<WCS_TaskInfo>().First(v => v.ID == obj.Data2.TaskNumber);
                if (taskInfo != null && taskInfo.AddrTo == obj.Entity.Code && taskInfo.Status == Entity.TaskStatus.Finish)
                {
                    WmsApi.CompleteTask(taskInfo.ID);
                    taskInfo.CancelTasks(db);

                }
               if (task == null)
                    return;
                obj.Data.TaskNumber = task.ID;
                obj.Data.GoodsStart = obj.Entity.Code.ToShort();
                obj.Data.GoodsEnd = task.AddrNext.ToShort();
                obj.Data.VoucherNo++;
                World.Log($"码垛位置{obj.Entity.Code}，任务{obj.Data2.TaskNumber}完成，写入PLC凭证号{obj.Data.VoucherNo + 1}", LogLevelEnum.Mid);
            });
        }

        public override bool Select(Device dev)
        {
            return dev.HasFlag(DeviceFlags.码垛工位);
        }
    }
}
